﻿using System;
using System.Runtime.InteropServices;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Input;
using System.Windows.Media.Imaging;

namespace WpfLiveBindingDemo
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();

            var c = new ImageToBitmapSourceConverter();
            image3.Source = (BitmapSource)c.Convert(Properties.Resources.EasyUAClient, typeof(BitmapSource), null, null);
            image2.Source = (BitmapSource)c.Convert(Properties.Resources.EasyUAClient, typeof(BitmapSource), null, null);
            image1.Source = (BitmapSource)c.Convert(Properties.Resources.EasyUAClient, typeof(BitmapSource), null, null);
        }

        private void automaticSubscriptionLinkLabel_MouseDown(object sender, MouseButtonEventArgs e)
        {
            SetHelpRichTextBox(Properties.Resources.MainWindow_automaticSubscriptionLinkLabel_LinkClicked_HelpText);
        }

        private void automaticReadLinkLabel_MouseDown(object sender, MouseButtonEventArgs e)
        {
            SetHelpRichTextBox(Properties.Resources.MainWindow_automaticReadLinkLabel_LinkClicked_HelpText);
        }

        private void readOnCustomEventLinkLabel_MouseDown(object sender, MouseButtonEventArgs e)
        {
            SetHelpRichTextBox(Properties.Resources.MainWindow_readOnCustomEventLinkLabel_LinkClicked_HelpText);
        }

        private void stringFormattingLinkLabel_MouseDown(object sender, MouseButtonEventArgs e)
        {
            SetHelpRichTextBox(Properties.Resources.MainWindow_stringFormattingLinkLabel_LinkClicked_HelpText);
        }

        private void changeBackgroundLinkLabel_MouseDown(object sender, MouseButtonEventArgs e)
        {
            SetHelpRichTextBox(Properties.Resources.MainWindow_changeBackgroundLinkLabel_LinkClicked_HelpText);
        }

        private void extendersLinkLabel_MouseDown(object sender, MouseButtonEventArgs e)
        {
            SetHelpRichTextBox(Properties.Resources.MainWindow_extendersLinkLabel_LinkClicked_HelpText);
        }

        private void bindingKindsLinkLabel_MouseDown(object sender, MouseButtonEventArgs e)
        {
            SetHelpRichTextBox(Properties.Resources.MainWindow_bindingKindsLinkLabel_LinkClicked_HelpText);
        }

        private void errorProviderLinkLabel_MouseDown(object sender, MouseButtonEventArgs e)
        {
            SetHelpRichTextBox(Properties.Resources.MainWindow_errorProviderLinkLabel_LinkClicked_HelpText);
        }

        private void writeSingleLinkLabel_MouseDown(object sender, MouseButtonEventArgs e)
        {
            SetHelpRichTextBox(Properties.Resources.MainWindow_writeSingleLinkLabel_LinkClicked_HelpText);
        }

        private void writeGroupLinkLabel_MouseDown(object sender, MouseButtonEventArgs e)
        {
            SetHelpRichTextBox(Properties.Resources.MainWindow_writeGroupLinkLabel_LinkClicked_HelpText);
        }

        private void subscribeAndWriteLinkLabel_MouseDown(object sender, MouseButtonEventArgs e)
        {
            SetHelpRichTextBox(Properties.Resources.MainWindow_subscribeAndWriteLinkLabel_LinkClicked_HelpText);
        }

        private void displayWriteErrorsLinkLabel_MouseDown(object sender, MouseButtonEventArgs e)
        {
            SetHelpRichTextBox(Properties.Resources.MainWindow_displayWriteErrorsLinkLabel_LinkClicked_HelpText);
        }

        private void SetHelpRichTextBox(string helpText)
        {
            helpRichTextBox.SelectAll();
            helpRichTextBox.Selection.Text = helpText;
            helpRichTextBox.CaretPosition = helpRichTextBox.CaretPosition.DocumentStart;
        }

        public static BitmapSource EasyUaClientBitmapSource = null;

    }


    /// <summary>
    /// Converts from <see cref="System.Drawing.Image"/> to <see cref="BitmapSource"/>.
    /// </summary>
    [ValueConversion(typeof(Image), typeof(BitmapSource))]
    internal class ImageToBitmapSourceConverter : IValueConverter
    {
        [DllImport("gdi32.dll")]
        private static extern bool DeleteObject(IntPtr hObject);

        #region IValueConverter Implementation

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            try
            {
                var image = value as System.Drawing.Image;
                if (image is null) return null;

                BitmapSource bitmapSource;
                using (var bitmap = new System.Drawing.Bitmap(image))
                {
                    IntPtr bmpPt = bitmap.GetHbitmap();
                    bitmapSource =
                        System.Windows.Interop.Imaging.CreateBitmapSourceFromHBitmap(
                            bmpPt,
                            IntPtr.Zero,
                            Int32Rect.Empty,
                            BitmapSizeOptions.FromEmptyOptions());

                    //freeze bitmapSource and clear memory to avoid memory leaks
                    bitmapSource.Freeze();
                    DeleteObject(bmpPt);
                }
                return bitmapSource;
            }
            // ReSharper disable once EmptyGeneralCatchClause
            catch { }
            return null;
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            return Binding.DoNothing;
        }

        #endregion

    }
}
